/**
 * 
 */
package net.bigeon.gclc.swt;

import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;

import net.bigeon.gclc.utils.AOutputForwardRunnable;
import net.bigeon.gclc.utils.PipedConsoleOutput;

/** The manager for console output to insert in a text.
 * 
 * @author Emmanuel Bigeon */
public final class ConsoleOutputManager implements ConsoleOutputDisplay {
    /** A runnable appending text to the content of a {@link Text} component.
     * 
     * @author Emmanuel Bigeon */
    private static class TextAppendingRunnable implements Runnable {
        /** The text to append on a line (possibly new). */
        private final String next;
        /** The {@link Text} component. */
        private final Text   text;

        /** Create the appending runnable.
         * 
         * @param text the component to update
         * @param next the text to append */
        public TextAppendingRunnable(final Text text, final String next) {
            this.text = text;
            this.next = next;
        }

        /* (non-Javadoc)
         * @see java.lang.Runnable#run() */
        @Override
        public void run() {
            final String initialText = text.getText();
            if (initialText != null && !initialText.isEmpty()) {
                text.append(System.lineSeparator());
            }
            text.append(next);
        }
    }

    /** The local implementation of the forwarding runnable.
     *
     * @author Emmanuel Bigeon */
    private static final class ToSWTConsoleForwardRunnable
            extends AOutputForwardRunnable {
        /** The running status. */
        private boolean                    running = true;
        /** The console output. */
        private final PipedConsoleOutput   out;
        /** The console output display. */
        private final ConsoleOutputDisplay display;
        /** The actual SWT component. */
        private final Widget               element;

        /** Create the forwarding runnable.
         * 
         * @param manager the manager
         * @param display the display
         * @param element the composite */
        public ToSWTConsoleForwardRunnable(final PipedConsoleOutput manager,
                                           final ConsoleOutputDisplay display,
                                           final Widget element) {
            super(manager);
            out = manager;
            this.display = display;
            this.element = element;
        }

        /* (non-Javadoc)
         * @see
         * net.bigeon.gclc.utils.AOutputForwardRunnable#forwardLine(java.lang.String) */
        @Override
        protected void forwardLine(final String m) {
            display.appendLine(m);
        }

        /* (non-Javadoc)
         * @see net.bigeon.gclc.utils.AOutputForwardRunnable#isRunning() */
        @Override
        protected boolean isRunning() {
            return running && !element.isDisposed();
        }

        /** Set the running status.
         * 
         * @param running the running to set */
        public void setRunning(final boolean running) {
            this.running = running;
        }

        /** Get the output.
         * 
         * @return the currently forwarded output */
        public PipedConsoleOutput getOuput() {
            return out;
        }
    }

    /** The SWT component displaying the output content. */
    private final Text                  text;
    /** The forwarding runnable. */
    private ToSWTConsoleForwardRunnable forward;
    /** The forwarding thread. */
    private Thread                      forwardThread;

    /** Create the manager.
     * 
     * @param text the text to display the output in. */
    public ConsoleOutputManager(final Text text) {
        super();
        this.text = text;
    }

    /* (non-Javadoc)
     * @see net.bigeon.gclc.swt.ConsoleOutputDisplay#appendLine(java.lang.String) */
    @Override
    public void appendLine(final String next) {
        text.getDisplay().syncExec(new TextAppendingRunnable(text, next));
    }

    /** Set the output.
     *
     * @param output the output to set */
    public void setManager(final PipedConsoleOutput output) {
        if (forward != null) {
            if (forward.getOuput() == output) {
                return;
            }
            forward.setRunning(false);
        }
        if (output == null) {
            forward = null;
            forwardThread = null;
            return;
        }
        forward = new ToSWTConsoleForwardRunnable(output, this, text);
        forwardThread = new Thread(forward, "gclcToSWT"); //$NON-NLS-1$
        forwardThread.start();
    }

    /** Get the current thread forwarding the console output to the SWT components.
     * 
     * @return the forwardThread */
    public Thread getForwardThread() {
        return forwardThread;
    }
}
